-- ============================================================
--   TABLE: VENDA
-- ============================================================
CREATE INDEX VENDA_08 ON VENDA(CDFILIAL, CDLOJA, CDCAIXA, NRSEQVENDA, DTENTRVENDA)
/

CREATE INDEX VENDA_09 ON VENDA(CDFILIAL, CDLOJA, CDCAIXA, NRSEQVENDA, DTVENDA)
/

CREATE INDEX VENDA_10 ON VENDA(CDFILIAL, CDLOJA, CDCAIXA, NRSEQVENDA, DTEMISSAONFCE)
/

-- ============================================================
--   TABLE: V_FILIALLOJACAIXA
-- ============================================================

CREATE VIEW V_FILIALLOJACAIXA
AS
SELECT LF.*, C.CDCAIXA, C.NMCAIXA
     FROM CAIXA C, (SELECT F.*, L.CDLOJA, L.NMLOJA
                      FROM LOJA L, (SELECT CDFILIAL, NMFILIAL
                                      FROM FILIAL) F
                     WHERE L.CDFILIAL = F.CDFILIAL) LF
    WHERE C.CDLOJA = LF.CDLOJA
      AND C.CDFILIAL = LF.CDFILIAL
/

-- =================================================================================
-- TABLE: METADIAREAL
-- =================================================================================

CREATE TABLE METADIAREAL
  (
    CDFILIAL     	VARCHAR2(4) 	NOT NULL,
    DATA         	DATE 		NOT NULL,
    CDANO        	NUMBER 		NOT NULL,
    VRMETADIA 	 	FLOAT(126) 	NOT NULL,
    VRTCKMED	 	FLOAT(126) 	NOT NULL,
    NRORG           	NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	VARCHAR2(1)    	DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METADIAREAL CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	DATE          	NULL,
    DTULTATU        	DATE          	NULL,
    NRORGINCLUSAO   	NUMBER       	NULL,
    CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
    NRORGULTATU     	NUMBER       	NULL,
    CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METADIAREAL PRIMARY KEY (CDFILIAL, DATA)
)
/

CREATE INDEX VND00699_FK ON METADIAREAL (CDFILIAL ASC)
/
		
ALTER TABLE METADIAREAL 
	ADD CONSTRAINT FKVND00699_METADIAREAL FOREIGN KEY (CDFILIAL) 
		REFERENCES FILIAL(CDFILIAL)
/
	
-- =================================================================================
-- TABLE: METAMODALIDADEREAL
-- =================================================================================
  
CREATE TABLE METAMODALIDADEREAL
 (
   CDFILIAL     	VARCHAR2(4) 	NOT NULL,
   DATA 		DATE 		NOT NULL,
   IDMODALIDADE 	VARCHAR2(3)	NOT NULL,
   VRMETAMODALIDADE	FLOAT(126) 	NOT NULL,
   NRORG           	NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
   IDATIVO	 	VARCHAR2(1)    	DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METAMODALIDADER CHECK (IDATIVO IN ('S','N')),
   DTINCLUSAO      	DATE          	NULL,
   DTULTATU        	DATE          	NULL,
   NRORGINCLUSAO   	NUMBER       	NULL,
   CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
   NRORGULTATU     	NUMBER       	NULL,
   CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METAMODALIDADEREAL PRIMARY KEY (CDFILIAL, DATA, IDMODALIDADE)
)
/

CREATE INDEX VND00700_FK ON METAMODALIDADEREAL (CDFILIAL ASC)
/

ALTER TABLE METAMODALIDADEREAL
	ADD CONSTRAINT FKVND00700_METAMODALIDADER FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/


-- =================================================================================
-- TABLE: METAHORAREAL
-- =================================================================================
 
CREATE TABLE METAHORAREAL
 (
   CDFILIAL     	VARCHAR2(4) 	NOT NULL,
   DATA 		DATE 		NOT NULL,
   HORA			NUMBER 		NOT NULL,
   VRMETAHORAACUM	FLOAT(126) 	NOT NULL,
   NRORG           	NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
   IDATIVO	 	VARCHAR2(1)    	DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METAHORAREAL CHECK (IDATIVO IN ('S','N')),
   DTINCLUSAO      	DATE          	NULL,
   DTULTATU        	DATE          	NULL,
   NRORGINCLUSAO   	NUMBER       	NULL,
   CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
   NRORGULTATU     	NUMBER       	NULL,
   CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METAHORAREAL PRIMARY KEY (CDFILIAL, DATA, HORA)
)
/

CREATE INDEX VND00701_FK ON METAHORAREAL (CDFILIAL ASC)
/

ALTER TABLE METAHORAREAL
	ADD CONSTRAINT FKVND00701METAHORAREAL FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: METAMES
-- =================================================================================

CREATE TABLE METAMES
  (
    CDFILIAL     	VARCHAR2(4) 	NOT NULL,
    CDMES		 NUMBER 	NOT NULL,
    CDANO 		 NUMBER 	NOT NULL,
    VRMES 		 FLOAT(126) 	NOT NULL,
    VRSEM 		 FLOAT(126) 	NOT NULL,
    NRORG           	 NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	 VARCHAR2(1)    DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METAMES CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	DATE          	NULL,
    DTULTATU        	DATE          	NULL,
    NRORGINCLUSAO   	NUMBER       	NULL,
    CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
    NRORGULTATU     	NUMBER       	NULL,
    CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METAMES PRIMARY KEY (CDFILIAL, CDMES, CDANO)
)
/

CREATE INDEX VND00702_FK ON METAMES (CDFILIAL ASC)
/

ALTER TABLE METAMES
	ADD CONSTRAINT FKVND00702METAMES FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: METADIA
-- =================================================================================

CREATE TABLE METADIA
  (
    CDFILIAL     	VARCHAR2(4) 	NOT NULL,
    NRDIA		NUMBER 		NOT NULL,
    VRPERCDIA    	FLOAT(126) 	NOT NULL,
    VRTICKETDIA  	FLOAT(126) 	NOT NULL,
    NRORG           	NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	VARCHAR2(1)     DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METADIA CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	DATE          	NULL,
    DTULTATU        	DATE          	NULL,
    NRORGINCLUSAO   	NUMBER       	NULL,
    CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
    NRORGULTATU     	NUMBER       	NULL,
    CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METADIA PRIMARY KEY (CDFILIAL, NRDIA)
)
/

CREATE INDEX VND00703_FK ON METADIA (CDFILIAL ASC)
/

ALTER TABLE METADIA
	ADD CONSTRAINT FKVND00703METADIA FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: METAMODALIDADEDIA
-- =================================================================================

CREATE TABLE METAMODALIDADEDIA
  (
    CDFILIAL     	 VARCHAR2(4) 	NOT NULL,
    IDMODALIDADE 	 VARCHAR2(3) 	NOT NULL,
    NRDIA        	 NUMBER 	NOT NULL,
    VRPERCMODALIDADE FLOAT(126) 	NOT NULL,
    NRORG           	 NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	 VARCHAR2(1)    DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METAMODALIDADED CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	 DATE          	NULL,
    DTULTATU        	 DATE          	NULL,
    NRORGINCLUSAO   	 NUMBER       	NULL,
    CDOPERINCLUSAO  	 VARCHAR2(12)  	NULL,
    NRORGULTATU     	 NUMBER       	NULL,
    CDOPERULTATU    	 VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METAMODALIDADEDIA PRIMARY KEY (CDFILIAL, IDMODALIDADE, NRDIA)
)
/

CREATE INDEX VND00704_FK ON METAMODALIDADEDIA (CDFILIAL ASC)
/

ALTER TABLE METAMODALIDADEDIA
	ADD CONSTRAINT FKVND00704METAMODALIDADEDIA FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: METADIAESPECIAL
-- =================================================================================

CREATE TABLE METADIAESPECIAL
  (
    CDFILIAL     	 VARCHAR2(4) 	NOT NULL,
    DSMETAESP	 	 VARCHAR2(200),
    DATAESP		 DATE 		NOT NULL,
    PERCCORRECAO 	 FLOAT(126) 	NOT NULL,
    NRORG           	 NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	 VARCHAR2(1)    DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METADIAESPECIAL CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	 DATE          	NULL,
    DTULTATU        	 DATE          	NULL,
    NRORGINCLUSAO   	 NUMBER       	NULL,
    CDOPERINCLUSAO  	 VARCHAR2(12)  	NULL,
    NRORGULTATU     	 NUMBER       	NULL,
    CDOPERULTATU    	 VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METADIAESPECIAL PRIMARY KEY (CDFILIAL, DATAESP)
)
/

CREATE INDEX VND00705_FK ON METADIAESPECIAL (CDFILIAL ASC)
/

ALTER TABLE METADIAESPECIAL
	ADD CONSTRAINT FKVND00705METADIAESPECIAL FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: METAHORA
-- =================================================================================

 CREATE TABLE METAHORA
  (
    CDFILIAL     	 VARCHAR2(4) 	NOT NULL,
    CDHORA		 NUMBER 	NOT NULL,
    VRPERCHORA 		 FLOAT(126) 	NOT NULL,
    NRORG                NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	 VARCHAR2(1)    DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_METAHORA CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	 DATE          	NULL,
    DTULTATU        	 DATE          	NULL,
    NRORGINCLUSAO   	 NUMBER       	NULL,
    CDOPERINCLUSAO  	 VARCHAR2(12)  	NULL,
    NRORGULTATU     	 NUMBER       	NULL,
    CDOPERULTATU    	 VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_METAHORA PRIMARY KEY (CDFILIAL, CDHORA)
)
/

CREATE INDEX VND00706_FK ON METAHORA (CDFILIAL ASC)
/

ALTER TABLE METAHORA
	ADD CONSTRAINT FKVND00706METAHORA FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: VENDADIAMODALIDADE
-- =================================================================================

CREATE TABLE VENDADIAMODALIDADE
  (
    CDFILIAL     	 VARCHAR2(4) 	NOT NULL,
    DTVENDA      	 DATE 		NOT NULL,
    IDMODALIDADE 	 VARCHAR2(7) 	NOT NULL,
    VRTOTAL      	 FLOAT(126) 	NOT NULL,
    NRORG           	 NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	 VARCHAR2(1)    DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_VENDADIAMODALID CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	 DATE          	NULL,
    DTULTATU        	 DATE          	NULL,
    NRORGINCLUSAO   	 NUMBER       	NULL,
    CDOPERINCLUSAO  	 VARCHAR2(12)  	NULL,
    NRORGULTATU     	 NUMBER       	NULL,
    CDOPERULTATU    	 VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_VENDADIAMODALIDADE PRIMARY KEY (CDFILIAL, DTVENDA, IDMODALIDADE)
)
/

CREATE INDEX VND00707_FK ON VENDADIAMODALIDADE (CDFILIAL ASC)
/

ALTER TABLE VENDADIAMODALIDADE
	ADD CONSTRAINT FKVND00707VENDADIAMODALIDADE FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: VENDADIATOTAL
-- =================================================================================

CREATE TABLE VENDADIATOTAL
  (
    CDFILIAL     	VARCHAR2(4) 	NOT NULL,
    DTVENDA      	DATE 		NOT NULL,
    NRVENDACON   	NUMBER 		NOT NULL,
    NRVENDACAN   	NUMBER 		NOT NULL,
    VRTICKETMED  	FLOAT (126)	NOT NULL,
    VRTOTAL      	FLOAT(126) 	NOT NULL,
    NRORG           	NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	VARCHAR2(1)    DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_VENDADIATOTAL CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	DATE          	NULL,
    DTULTATU        	DATE          	NULL,
    NRORGINCLUSAO   	NUMBER       	NULL,
    CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
    NRORGULTATU     	NUMBER       	NULL,
    CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_VENDADIATOTAL PRIMARY KEY (CDFILIAL, DTVENDA)
)
/

CREATE INDEX VND00708_FK ON VENDADIATOTAL (CDFILIAL ASC)
/

ALTER TABLE VENDADIATOTAL
	ADD CONSTRAINT FKVND00708VENDADIATOTAL FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: VENDAHORATOTAL
-- =================================================================================

CREATE TABLE VENDAHORATOTAL
  (
    CDFILIAL  		VARCHAR2(4) 	NOT NULL,
    DTVENDA   		DATE 		NOT NULL,
    VRTOTAL   		FLOAT(126) 	NOT NULL,
    VRTOTALACUMULADO 	FLOAT(126) 	NOT NULL,
    HORAVENDA 		NUMBER 		NOT NULL,
    NRORG           	NUMBER       	DEFAULT SYS_CONTEXT('nrorg_ctx','nrorg')  NOT NULL,
    IDATIVO	 	VARCHAR2(1)     DEFAULT 'S'	NULL,
		CONSTRAINT CKC_IDATIVO_VENDAHORATOTAL CHECK (IDATIVO IN ('S','N')),
    DTINCLUSAO      	DATE          	NULL,
    DTULTATU        	DATE          	NULL,
    NRORGINCLUSAO   	NUMBER       	NULL,
    CDOPERINCLUSAO  	VARCHAR2(12)  	NULL,
    NRORGULTATU     	NUMBER       	NULL,
    CDOPERULTATU    	VARCHAR2(12)  	NULL,			
		CONSTRAINT PK_VENDAHORATOTAL PRIMARY KEY (CDFILIAL, DTVENDA, HORAVENDA)
)
/

CREATE INDEX VND00709_FK ON VENDAHORATOTAL (CDFILIAL ASC)
/

ALTER TABLE VENDAHORATOTAL
	ADD CONSTRAINT FKVND00709VENDAHORATOTAL FOREIGN KEY (CDFILIAL)
		REFERENCES FILIAL(CDFILIAL)
/

-- =================================================================================
-- TABLE: VENDA
-- ================================================================================= 

ALTER TABLE VENDA ADD IDCONTVEND VARCHAR2(1) DEFAULT 'N'
/

-- ============================================================
--   TABLE: METAMES
-- ============================================================

ALTER TABLE METAMES ADD VRMETAESPECIAL FLOAT(126) DEFAULT 0   
/


-- ============================================================
--   TABLE: METAHORA
-- ============================================================

ALTER TABLE METAHORA DROP CONSTRAINT PK_METAHORA
/

ALTER TABLE METAHORA DROP CONSTRAINT FKVND00706METAHORA
/

-- ============================================================
--   TABLE: METAMODALIDADEDIA
-- ============================================================
  
ALTER TABLE METAMODALIDADEDIA DROP CONSTRAINT PK_METAMODALIDADEDIA
/

ALTER TABLE METAMODALIDADEDIA DROP CONSTRAINT FKVND00704METAMODALIDADEDIA
/

-- ============================================================
--   TABLE: METADIA
-- ============================================================
ALTER TABLE METADIA DROP CONSTRAINT PK_METADIA
/

ALTER TABLE METADIA DROP CONSTRAINT FKVND00703METADIA
/


ALTER TABLE METADIA ADD CDMES NUMBER DEFAULT 1 NOT NULL
/
  
ALTER TABLE METADIA ADD CDANO NUMBER DEFAULT 2018 NOT NULL
	   
/
	   
ALTER TABLE METADIA
	ADD CONSTRAINT FKVND00703METADIA FOREIGN KEY (CDFILIAL, CDMES, CDANO)
   		REFERENCES METAMES (CDFILIAL, CDMES, CDANO)
/

ALTER TABLE METADIA ADD CONSTRAINT PK_METADIA PRIMARY KEY (CDFILIAL, NRDIA, CDMES, CDANO)
/  
	   
-- ============================================================
--   TABLE: METAMODALIDADEDIA
-- ============================================================
   
ALTER TABLE METAMODALIDADEDIA ADD CDMES NUMBER NOT NULL
/
  
ALTER TABLE METAMODALIDADEDIA ADD CDANO NUMBER NOT NULL
/
	   
ALTER TABLE METAMODALIDADEDIA
	ADD CONSTRAINT FKVND00704METAMODALIDADEDIA FOREIGN KEY (CDFILIAL, CDMES, CDANO)
   		REFERENCES METAMES (CDFILIAL, CDMES, CDANO)
/

ALTER TABLE METAMODALIDADEDIA
ADD CONSTRAINT PK_METAMODALIDADEDIA PRIMARY KEY (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO)
/
   
-- ============================================================
--   TABLE: METAHORA
-- ============================================================
   
ALTER TABLE METAHORA ADD NRDIA NUMBER NOT NULL
/
  
ALTER TABLE METAHORA ADD CDMES NUMBER NOT NULL
/

ALTER TABLE METAHORA ADD CDANO NUMBER NOT NULL

/

ALTER TABLE METAHORA
	ADD CONSTRAINT PK_METAHORA PRIMARY KEY (CDFILIAL, CDHORA, NRDIA, CDMES, CDANO)
/
	   
ALTER TABLE METAHORA
	ADD CONSTRAINT FKVND00706METAHORA FOREIGN KEY (CDFILIAL, CDMES, CDANO)   
   		REFERENCES METAMES (CDFILIAL, CDMES, CDANO)
/
   
ALTER TABLE METAHORA
	ADD CONSTRAINT FKMETAHORAMETADIA FOREIGN KEY (CDFILIAL, NRDIA, CDMES, CDANO)
   		REFERENCES METADIA (CDFILIAL, NRDIA, CDMES, CDANO)
/

/* PROCEDURE */
  CREATE OR REPLACE PROCEDURE VENDA_HORA 
  (
	V_CDFILIAL IN VARCHAR2,  V_NRORG NUMBER
  )
AS 
  V_VENDAHORA       FLOAT; 
  V_VENDAACUMULADA  FLOAT;
  V_CONTHORA        NUMBER;
BEGIN

	DELETE
	  FROM VENDAHORATOTAL
     WHERE TRUNC(DTVENDA) = TRUNC(sysdate) AND CDFILIAL = V_CDFILIAL;
		
	V_CONTHORA := 0;
	V_VENDAACUMULADA := 0;	
	
	WHILE V_CONTHORA <= 23 LOOP
	
		V_VENDAHORA := 0;
		
		FOR C_VENDA IN
		(	SELECT CDFILIAL, NMFILIAL, IDORIGEMVENDA, DTENTRVENDA, NRORG,
			   SUM(TOTALVEND) TOTALVEND,
			   TRUNC(SUM(TICKETMEDIO) / SUM(DECODE(VRQTVENDACONC, 0, 1, VRQTVENDACONC)), 2) TICKETMEDIO,
			   SUM(VRQTVENDACONC) VRQTVENDACONC, SUM(VRQTVENDACANC) VRQTVENDACANC
			FROM  (SELECT PARCIAL.CDFILIAL, PARCIAL.NMFILIAL, PARCIAL.IDORIGEMVENDA, PARCIAL.DTENTRVENDA, PARCIAL.NRORG,
						SUM(DECODE(PARCIAL.IDSITUVENDA, 'C', 0, PARCIAL.TOTALVEND)) TOTALVEND,
						TRUNC(SUM(PARCIAL.TICKETMEDIO)) AS TICKETMEDIO,
						SUM(COUNT(DISTINCT VRQTVENDACONC)) OVER (PARTITION BY CDFILIAL, IDORIGEMVENDA) VRQTVENDACONC,
						SUM(COUNT(DISTINCT VRQTVENDACANC)) OVER (PARTITION BY CDFILIAL, IDORIGEMVENDA) VRQTVENDACANC
				   FROM (SELECT X.CDFILIAL, X.NMFILIAL, SUBSTR(X.IDORIGEMVENDA, 0, 3) AS  IDORIGEMVENDA, X.DTENTRVENDA, X.NRORG,
								DECODE(IDSITUVENDA, 'C', 0, TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+NVL(IT.VRUNITVENDCL,0)),2) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TOTALVEND,
								DECODE(IDSITUVENDA, 'C', 0, TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+NVL(IT.VRUNITVENDCL,0)),2) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TICKETMEDIO,
								X.IDSITUVENDA,
								DECODE(IDSITUVENDA, 'O', X.CDFILIAL||X.CDCAIXA||X.NRSEQVENDA) VRQTVENDACONC,
								DECODE(IDSITUVENDA, 'C', X.CDFILIAL||X.CDCAIXA||X.NRSEQVENDA) VRQTVENDACANC
						   FROM (SELECT DISTINCT V.CDFILIAL, 
										V.CDCAIXA,
										V.NRSEQVENDA,
										FLC.NMFILIAL,
										V.IDORIGEMVENDA,
										V.NRORG,
										V.DTENTRVENDA,
										V.IDSITUVENDA
								   FROM VENDA V, (SELECT DISTINCT CDFILIAL, NMFILIAL, CDLOJA, CDCAIXA FROM V_FILIALLOJACAIXA WHERE CDFILIAL = V_CDFILIAL) FLC
								  WHERE V.CDCAIXA   = FLC.CDCAIXA
									AND V.CDLOJA      = FLC.CDLOJA
									AND V.CDFILIAL    = FLC.CDFILIAL
									AND TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) = V_CONTHORA
									AND V.DTENTRVENDA = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY')
									AND V.IDSITUVENDA IN ('O', 'C')
									ORDER BY V.NRSEQVENDA) X LEFT JOIN ITEMVENDA IT
																	ON X.CDFILIAL = IT.CDFILIAL
																   AND X.CDCAIXA    = IT.CDCAIXA
																   AND X.NRSEQVENDA = IT.NRSEQVENDA) PARCIAL
				  GROUP BY PARCIAL.CDFILIAL, PARCIAL.NMFILIAL, PARCIAL.IDORIGEMVENDA, PARCIAL.DTENTRVENDA, PARCIAL.NRORG) FINAL
			GROUP BY CDFILIAL, NMFILIAL, IDORIGEMVENDA, DTENTRVENDA, NRORG
			ORDER BY CDFILIAL) LOOP	
				
				V_VENDAHORA := C_VENDA.TOTALVEND;
				V_VENDAACUMULADA := V_VENDAACUMULADA + C_VENDA.TOTALVEND;
				
		END LOOP;
		
		INSERT INTO VENDAHORATOTAL
		  (CDFILIAL, DTVENDA, HORAVENDA, VRTOTAL, VRTOTALACUMULADO, NRORG )
		  VALUES
		  (V_CDFILIAL, SYSDATE, V_CONTHORA,
			V_VENDAHORA, V_VENDAACUMULADA, V_NRORG);	
			
		V_CONTHORA := V_CONTHORA + 1;
	END LOOP;	
	
	COMMIT;
END;
/


/* PROCEDURE */
  create or replace PROCEDURE CALCULA_METAS_RM
   (
	V_CDFILIAL IN VARCHAR2,  V_NRORG NUMBER
   )
AS
  TYPE tNome IS VARRAY(7) OF VARCHAR(25);
  V_META		    NUMBER;
  V_CDMES           NUMBER;
  V_CDANO           NUMBER;
  V_VALORMETA       FLOAT;
  V_VALORTCKMED     FLOAT;
  V_METAACUM        FLOAT;
  V_CONTHORA        NUMBER;
  V_NRDIA           NUMBER;  
  V_DIAESPECIAL     FLOAT;
  V_VALORTEMP       FLOAT;
  V_DIASDASEMANA	NUMBER;
BEGIN

	V_CDANO 	:= TO_NUMBER(TO_CHAR(sysdate, 'YYYY'));	
			
	FOR k IN 1..12 LOOP
		V_CDMES := k;
		/* Verifica se h alguma meta parametrizada para cada unidade */
		BEGIN
			SELECT COUNT(CDFILIAL) INTO V_META FROM METAMES WHERE CDFILIAL = V_CDFILIAL AND CDMES = V_CDMES AND NRORG = V_NRORG;
		EXCEPTION
		WHEN NO_DATA_FOUND THEN
			V_META := 0;
		END;
		  
		/* Caso j exista meta parametrizada, realiza o calculo do valor real que ser tido como META 
		   Caso no exista, realiza a estimativa de meta.	*/  
		IF (V_META > 0) THEN	
			UPDATE METAMES SET VRMETAESPECIAL = 0
			WHERE CDFILIAL = V_CDFILIAL AND CDMES = V_CDMES AND CDANO = V_CDANO AND NRORG = V_NRORG;
			FOR j IN 1..7 LOOP	
				V_NRDIA := j;
				/* Caso haja a meta, pega as informaes necessrias para realizar o clculo */
				FOR C_METAS IN (SELECT M.CDFILIAL, M.VRMES * (MD.VRPERCDIA/100) AS METADIAREAL, MD.VRTICKETDIA AS TICKETMEDIO, M.NRORG
								  FROM METAMES M
								  JOIN METADIA MD
								  ON M.CDFILIAL    = MD.CDFILIAL
								  AND MD.NRDIA     = V_NRDIA
								  AND MD.CDMES      = V_CDMES
								  AND MD.CDANO      = V_CDANO
								  WHERE M.CDFILIAL = V_CDFILIAL
								  AND M.CDMES      = V_CDMES
								  AND M.CDANO      = V_CDANO) 
				LOOP

					
					SELECT COUNT(*)
						INTO V_DIASDASEMANA
					FROM DATA
					WHERE TO_CHAR(DTDATA, 'D') = V_NRDIA
						AND TO_CHAR(DTDATA, 'MM') = V_CDMES
						AND TO_CHAR(DTDATA, 'YYYY') = V_CDANO;
					
					V_VALORTCKMED  := C_METAS.TICKETMEDIO;
					V_VALORMETA    := C_METAS.METADIAREAL/V_DIASDASEMANA;
					
					FOR C_DATAS IN (
						SELECT DTDATA AS DATAINSERT
						FROM DATA
						WHERE TO_CHAR(DTDATA, 'D') = V_NRDIA
						AND TO_CHAR(DTDATA, 'MM') = V_CDMES
						AND TO_CHAR(DTDATA, 'YYYY') = V_CDANO) LOOP
						
						/* Limpa as tabelas de Meta Calculada */
						DELETE FROM METADIAREAL
						WHERE TRUNC(DATA) = TRUNC(C_DATAS.DATAINSERT) AND CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG;
						
						DELETE FROM METAMODALIDADEREAL
						WHERE TRUNC(DATA) = TRUNC(C_DATAS.DATAINSERT) AND CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG;
						
						DELETE FROM METAHORAREAL
						WHERE TRUNC(DATA) = TRUNC(C_DATAS.DATAINSERT) AND CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG;
						
						/* Verifica se a data atual  um dia especial */
						BEGIN
							SELECT PERCCORRECAO INTO V_DIAESPECIAL FROM METADIAESPECIAL 
							WHERE CDFILIAL = V_CDFILIAL AND TRUNC(DATAESP) = TRUNC(C_DATAS.DATAINSERT);
						EXCEPTION
						WHEN NO_DATA_FOUND THEN
							V_DIAESPECIAL := 0;
						END;
						
						/* Caso seja, aplica o fator de correo */
						IF (V_DIAESPECIAL != 0) THEN
							V_VALORTEMP := (V_DIAESPECIAL/100 * V_VALORMETA);
							V_VALORMETA := V_VALORMETA + V_VALORTEMP;
							V_VALORTCKMED := V_VALORTCKMED + (V_DIAESPECIAL/100 * V_VALORTCKMED);
							
							UPDATE METAMES SET VRMETAESPECIAL = VRMETAESPECIAL + V_VALORTEMP
							WHERE CDFILIAL = V_CDFILIAL AND CDMES = V_CDMES AND CDANO = V_CDANO AND NRORG = V_NRORG;
			
						END IF;
						
						INSERT INTO METADIAREAL (CDFILIAL, DATA, CDANO, VRMETADIA, VRTCKMED, NRORG)
						VALUES (V_CDFILIAL, C_DATAS.DATAINSERT, V_CDANO, V_VALORMETA, V_VALORTCKMED, V_NRORG);
						
						/* Salvando a meta de cada modalidade utilizada */
						FOR C_METAMODALIDADE IN (SELECT IDMODALIDADE, VRPERCMODALIDADE
												 FROM METAMODALIDADEDIA
												 WHERE CDFILIAL = V_CDFILIAL
												 AND NRDIA 		= V_NRDIA
												 AND CDMES      = V_CDMES
												 AND CDANO      = V_CDANO)
						LOOP
							INSERT INTO METAMODALIDADEREAL (CDFILIAL, DATA, IDMODALIDADE, VRMETAMODALIDADE, NRORG)
							VALUES (V_CDFILIAL, C_DATAS.DATAINSERT, C_METAMODALIDADE.IDMODALIDADE, 
									V_VALORMETA * (C_METAMODALIDADE.VRPERCMODALIDADE/100), V_NRORG );
						END LOOP;
						
						V_CONTHORA := 0;
						V_METAACUM := 0;
						
						WHILE V_CONTHORA <= 23 LOOP
								/* Salvando a meta de cada modalidade utilizada */
								FOR C_METAHORA IN ( SELECT CDFILIAL, VRPERCHORA
													FROM METAHORA
													WHERE CDFILIAL  = V_CDFILIAL
													AND CDHORA 		= V_CONTHORA
													AND NRDIA 		= V_NRDIA	
													AND CDMES   	= V_CDMES
													AND CDANO   	= V_CDANO)
								LOOP
									V_METAACUM := V_METAACUM + (V_VALORMETA * (C_METAHORA.VRPERCHORA/100));	
								END LOOP;
								
								INSERT INTO METAHORAREAL (CDFILIAL, DATA, HORA, VRMETAHORAACUM, NRORG)
								VALUES (V_CDFILIAL, C_DATAS.DATAINSERT, V_CONTHORA, V_METAACUM, V_NRORG);
										
								V_CONTHORA := V_CONTHORA + 1;
						END LOOP;	
						
					END LOOP;			
					
					
				END LOOP;
			END LOOP;
		END IF;
	END LOOP;
	COMMIT;
END;
/

/* PROCEDURE */
  CREATE OR REPLACE PROCEDURE ESTIMA_META 
  (
	V_CDFILIAL IN VARCHAR2,  V_NRORG NUMBER, V_PERCACRES FLOAT
  )
AS 
  TYPE tDias  IS VARRAY(7) OF FLOAT;
  TYPE tHoras IS VARRAY(24) OF FLOAT;
  V_NRDIA           NUMBER; 
  V_CDMES           NUMBER;
  V_CDANO           NUMBER;
  V_CONTHORA        NUMBER;
  V_CONTADOR        NUMBER;
  V_VNDREALIZADAS   NUMBER;
  V_DIASDASEMANA	NUMBER;
  V_TOT_VND_DIA     FLOAT;
  V_EST_CMD         FLOAT;
  V_EST_TAA         FLOAT;
  V_EST_DLV         FLOAT;
  V_EST_BAL         FLOAT;
  V_EST_MES         FLOAT;
  V_EST_OUT         FLOAT;
  V_VND_PROV        FLOAT;
  V_VND_PROV2       FLOAT;
  V_VENDASEMANA		FLOAT;
  V_TOTVENDAMES     FLOAT;
  V_VALORES		    tDias;
  V_HORAS           tHoras;
BEGIN

	V_CDANO 	:= TO_NUMBER(TO_CHAR(sysdate, 'YYYY'));
	
	DELETE FROM METAHORA WHERE CDFILIAL = V_CDFILIAL AND CDANO = V_CDANO;

	DELETE FROM METAMODALIDADEDIA WHERE CDFILIAL = V_CDFILIAL AND CDANO = V_CDANO;

	DELETE FROM METADIA WHERE CDFILIAL = V_CDFILIAL AND CDANO = V_CDANO;

	DELETE FROM METAMES WHERE CDFILIAL = V_CDFILIAL AND CDANO = V_CDANO;
	
	FOR C_FERIADO IN
		(SELECT DSFERIFILI, DTFERIFILI FROM FERIFILI WHERE CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG AND TO_NUMBER(TO_CHAR(DTFERIFILI, 'YYYY')) =  V_CDANO) LOOP	
		
		BEGIN
				SELECT COUNT(CDFILIAL) INTO V_CONTADOR FROM METADIAESPECIAL 
				WHERE CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG AND TRUNC(DATAESP) = TRUNC(C_FERIADO.DTFERIFILI);
		EXCEPTION
		WHEN NO_DATA_FOUND THEN
			V_CONTADOR := 0;
		END;
		
		IF V_CONTADOR = 0 THEN

			INSERT INTO METADIAESPECIAL (CDFILIAL, DATAESP, PERCCORRECAO, DSMETAESP, NRORG)
			VALUES (V_CDFILIAL, C_FERIADO.DTFERIFILI, V_PERCACRES, C_FERIADO.DSFERIFILI, V_NRORG);						
			
		END IF;		
	/* END FOR */			
	END LOOP;
	
	FOR k IN 1..12 LOOP
		V_CDMES 	:= k;	
		V_VALORES   := tDias(0, 0, 0, 0, 0, 0, 0);
		V_VENDASEMANA := 0;
		V_TOTVENDAMES := 0;
		
		INSERT INTO METAMES (CDFILIAL, CDMES, CDANO, VRMES, VRSEM, NRORG)
		VALUES (V_CDFILIAL, V_CDMES, V_CDANO, 0, 0, V_NRORG);
		
		FOR j IN 1..7 LOOP
			V_NRDIA     := j;
			
			V_HORAS := tHoras(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
			
			V_CONTADOR  := 0;
			
			V_VNDREALIZADAS := 0;
			V_DIASDASEMANA := 0;
			V_VND_PROV2 := 0;
			
			V_VND_PROV := 0;
			V_EST_CMD  := 0;
			V_EST_TAA  := 0;
			V_EST_DLV  := 0;
			V_EST_BAL  := 0;
			V_EST_MES  := 0;
			V_EST_OUT  := 0;		

			/* Calcula previso de venda para o dia (Geral) */
			FOR C_VENDA IN
				(   SELECT CDFILIAL, NMFILIAL, IDORIGEMVENDA, DTENTRVENDA, NRORG,
						 NVL(SUM(TOTALVEND), 0) TOTALVEND, HORAVEND,
						 NVL(TRUNC(SUM(TICKETMEDIO) / SUM(DECODE(VRQTVENDACONC, 0, 1, VRQTVENDACONC)), 2), 0) TICKETMEDIO,
						 SUM(VRQTVENDACONC) VRQTVENDACONC, SUM(VRQTVENDACANC) VRQTVENDACANC
					FROM  (SELECT PARCIAL.CDFILIAL, PARCIAL.NMFILIAL, PARCIAL.IDORIGEMVENDA, PARCIAL.DTENTRVENDA, PARCIAL.NRORG,
								  SUM(DECODE(PARCIAL.IDSITUVENDA, 'C', 0, PARCIAL.TOTALVEND)) TOTALVEND, PARCIAL.HORAVEND,
								  TRUNC(SUM(PARCIAL.TICKETMEDIO)) AS TICKETMEDIO,
								  SUM(COUNT(DISTINCT VRQTVENDACONC)) OVER (PARTITION BY CDFILIAL, IDORIGEMVENDA) VRQTVENDACONC,
								  SUM(COUNT(DISTINCT VRQTVENDACANC)) OVER (PARTITION BY CDFILIAL, IDORIGEMVENDA) VRQTVENDACANC
							 FROM (SELECT X.CDFILIAL, X.NMFILIAL, SUBSTR(X.IDORIGEMVENDA, 0, 3) AS  IDORIGEMVENDA, X.DTENTRVENDA, X.NRORG,
										  DECODE(IDSITUVENDA, 'C', 0, TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+NVL(IT.VRUNITVENDCL,0)),2) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TOTALVEND,
										  DECODE(IDSITUVENDA, 'C', 0, TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+NVL(IT.VRUNITVENDCL,0)),2) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TICKETMEDIO,
										  X.IDSITUVENDA, X.HORAVEND,
										  DECODE(IDSITUVENDA, 'O', X.CDFILIAL||X.CDCAIXA||X.NRSEQVENDA) VRQTVENDACONC,
										  DECODE(IDSITUVENDA, 'C', X.CDFILIAL||X.CDCAIXA||X.NRSEQVENDA) VRQTVENDACANC
								  FROM (SELECT DISTINCT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, FLC.NMFILIAL, TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) HORAVEND,
												  V.IDORIGEMVENDA, V.NRORG, V.DTENTRVENDA, V.IDSITUVENDA
											 FROM VENDA V, (SELECT DISTINCT CDFILIAL, NMFILIAL, CDLOJA, CDCAIXA
															  FROM V_FILIALLOJACAIXA WHERE CDFILIAL = V_CDFILIAL) FLC
											WHERE V.CDCAIXA   = FLC.CDCAIXA
											  AND V.CDLOJA    = FLC.CDLOJA
											  AND V.CDFILIAL  = FLC.CDFILIAL
											  AND V_NRDIA     = TO_CHAR(V.DTENTRVENDA,'D')
											  AND V_CDMES     = TO_NUMBER(TO_CHAR(V.DTENTRVENDA, 'MM'))
											  AND (V_CDANO-1) = TO_NUMBER(TO_CHAR(V.DTENTRVENDA, 'YYYY'))
											  AND V.IDSITUVENDA IN ('O', 'C')
											  ORDER BY 1, 2, 3) X 
									LEFT JOIN ITEMVENDA IT
									ON X.CDFILIAL = IT.CDFILIAL
								  AND X.CDCAIXA    = IT.CDCAIXA
								  AND X.NRORG      = IT.NRORG
								  AND X.NRSEQVENDA = IT.NRSEQVENDA
                                 ORDER BY 1, 2, 3) PARCIAL
						  GROUP BY PARCIAL.CDFILIAL, PARCIAL.NMFILIAL, PARCIAL.IDORIGEMVENDA, PARCIAL.DTENTRVENDA, PARCIAL.HORAVEND, PARCIAL.NRORG
						  ORDER BY 1, 2, 3) FINAL
				  GROUP BY CDFILIAL, NMFILIAL, IDORIGEMVENDA, DTENTRVENDA, HORAVEND, NRORG
				  ORDER BY CDFILIAL
				) LOOP	
				/* Aplica um valor X a mais no valor vendido */
				V_VNDREALIZADAS := C_VENDA.TOTALVEND + ((V_PERCACRES/100)*C_VENDA.TOTALVEND);
				V_CONTADOR := V_CONTADOR + 1;
				V_HORAS(C_VENDA.HORAVEND+1) := V_HORAS(C_VENDA.HORAVEND+1) + V_VNDREALIZADAS;
				
				IF C_VENDA.IDORIGEMVENDA = 'CMD' THEN
					V_EST_CMD  := V_EST_CMD + V_VNDREALIZADAS;
				ELSIF C_VENDA.IDORIGEMVENDA = 'TAA' THEN 
					V_EST_TAA  := V_EST_TAA + V_VNDREALIZADAS;
				ELSIF C_VENDA.IDORIGEMVENDA = 'DLV' THEN 
					V_EST_DLV  := V_EST_DLV + V_VNDREALIZADAS;
				ELSIF C_VENDA.IDORIGEMVENDA = 'BAL' THEN 
					V_EST_BAL  := V_EST_BAL + V_VNDREALIZADAS;
				ELSIF C_VENDA.IDORIGEMVENDA = 'MES' THEN 
					V_EST_MES  := V_EST_MES + V_VNDREALIZADAS;
				ELSE 
					V_EST_OUT  := V_EST_OUT + V_VNDREALIZADAS;
				END IF;	
				
			/* END FOR */			
			END LOOP;
			
			IF V_CONTADOR > 0 THEN
				SELECT COUNT(*)
					INTO V_DIASDASEMANA
				FROM DATA
				WHERE TO_CHAR(DTDATA, 'D') = V_NRDIA
					AND TO_CHAR(DTDATA, 'MM') = V_CDMES
					AND TO_CHAR(DTDATA, 'YYYY') = V_CDANO-1;
				/* Realiza a mdia das vendas somadas de um determinado dia. */
				V_VNDREALIZADAS := (V_EST_CMD + V_EST_TAA + V_EST_DLV + V_EST_BAL + V_EST_MES + V_EST_OUT)/ V_DIASDASEMANA;
				SELECT COUNT(*)
					INTO V_DIASDASEMANA
				FROM DATA
				WHERE TO_CHAR(DTDATA, 'D') = V_NRDIA
					AND TO_CHAR(DTDATA, 'MM') = V_CDMES
					AND TO_CHAR(DTDATA, 'YYYY') = V_CDANO;
				/* Nesse momento, a varivel V_VNDREALIZADAS est com uma mdia de venda por dia da semana. */
				IF(V_VNDREALIZADAS > 0) THEN 
					V_EST_CMD  := ((V_EST_CMD/V_DIASDASEMANA) * 100) / V_VNDREALIZADAS;
					V_EST_TAA  := ((V_EST_TAA/V_DIASDASEMANA) * 100) / V_VNDREALIZADAS;
					V_EST_DLV  := ((V_EST_DLV/V_DIASDASEMANA) * 100) / V_VNDREALIZADAS;
					V_EST_BAL  := ((V_EST_BAL/V_DIASDASEMANA) * 100) / V_VNDREALIZADAS;
					V_EST_MES  := ((V_EST_MES/V_DIASDASEMANA) * 100) / V_VNDREALIZADAS;
					V_EST_OUT  := ((V_EST_OUT/V_DIASDASEMANA) * 100) / V_VNDREALIZADAS;
				ELSE
					V_EST_CMD  := 0;
					V_EST_TAA  := 0;
					V_EST_DLV  := 0;
					V_EST_BAL  := 0;
					V_EST_MES  := 0;
					V_EST_OUT  := 0;
				END IF;
			END IF;
			
			/* Atribuo ao array de dias da semana o valor que devo vender em um determinado dia da semana */
			V_VALORES(V_NRDIA) := V_VNDREALIZADAS;
			
			V_VENDASEMANA := V_VENDASEMANA + V_VNDREALIZADAS;
			V_TOTVENDAMES := V_TOTVENDAMES + (V_VNDREALIZADAS * V_DIASDASEMANA);

			V_CONTADOR := 0;	
			
			/* Confere se existe algum registro na METADIA e METAHORA. Se no houver, insere. 
			   Se no houver nenhum registro em METADIA  entendido que em METAHORA tambm no existe. */
			BEGIN
				SELECT COUNT(CDFILIAL) INTO V_CONTADOR FROM METADIA WHERE NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_CONTADOR := 0;
			END;
			
			IF V_CONTADOR = 0 THEN
	
				INSERT INTO METADIA (CDFILIAL, NRDIA, CDMES, CDANO, VRPERCDIA, VRTICKETDIA, NRORG)
				VALUES (V_CDFILIAL, V_NRDIA, V_CDMES, V_CDANO, TRUNC(100/7,2), 0, V_NRORG);						
				
				V_CONTHORA := 0;
				WHILE V_CONTHORA <= 23 LOOP
				
					INSERT INTO METAHORA (CDFILIAL, CDHORA, NRDIA, CDMES, CDANO, VRPERCHORA, NRORG)
					VALUES (V_CDFILIAL, V_CONTHORA, V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
					
					V_CONTHORA := V_CONTHORA + 1;
				END LOOP;
			END IF;
			
			V_CONTADOR := 0;
			
			/* Confere se existe algum registro na METAMODALIDADE. Se no houver, insere. */
			BEGIN
				SELECT COUNT(CDFILIAL) INTO V_CONTADOR FROM METAMODALIDADEDIA WHERE NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND CDFILIAL = V_CDFILIAL AND NRORG = V_NRORG;
			EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_CONTADOR := 0;
			END;	
			
			IF V_CONTADOR = 0 THEN	
					
				INSERT INTO METAMODALIDADEDIA (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO, VRPERCMODALIDADE, NRORG)
				VALUES (V_CDFILIAL, 'CMD', V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
				
				INSERT INTO METAMODALIDADEDIA (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO, VRPERCMODALIDADE, NRORG)
				VALUES (V_CDFILIAL, 'TAA', V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
				
				INSERT INTO METAMODALIDADEDIA (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO, VRPERCMODALIDADE, NRORG)
				VALUES (V_CDFILIAL, 'DLV', V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
				
				INSERT INTO METAMODALIDADEDIA (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO, VRPERCMODALIDADE, NRORG)
				VALUES (V_CDFILIAL, 'BAL', V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
				
				INSERT INTO METAMODALIDADEDIA (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO, VRPERCMODALIDADE, NRORG)
				VALUES (V_CDFILIAL, 'MES', V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
				
				INSERT INTO METAMODALIDADEDIA (CDFILIAL, IDMODALIDADE, NRDIA, CDMES, CDANO, VRPERCMODALIDADE, NRORG)
				VALUES (V_CDFILIAL, 'OUT', V_NRDIA, V_CDMES, V_CDANO, 0, V_NRORG);
			
			END IF;	
			
			/* Atualiza a estimativa da modalidade CMD */
			UPDATE METAMODALIDADEDIA SET VRPERCMODALIDADE = V_EST_CMD
			WHERE CDFILIAL = V_CDFILIAL AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND IDMODALIDADE = 'CMD' AND NRORG = V_NRORG;
			
			/* Atualiza a estimativa da modalidade TAA */
			UPDATE METAMODALIDADEDIA SET VRPERCMODALIDADE = V_EST_TAA
			WHERE CDFILIAL = V_CDFILIAL AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND IDMODALIDADE = 'TAA' AND NRORG = V_NRORG;
			
			/* Atualiza a estimativa da modalidade DLV */
			UPDATE METAMODALIDADEDIA SET VRPERCMODALIDADE = V_EST_DLV
			WHERE CDFILIAL = V_CDFILIAL AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND IDMODALIDADE = 'DLV' AND NRORG = V_NRORG;
			
			/* Atualiza a estimativa da modalidade BAL */	
			UPDATE METAMODALIDADEDIA SET VRPERCMODALIDADE = V_EST_BAL
			WHERE CDFILIAL = V_CDFILIAL AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND IDMODALIDADE = 'BAL' AND NRORG = V_NRORG;
			
			/* Atualiza a estimativa da modalidade MES */
			UPDATE METAMODALIDADEDIA SET VRPERCMODALIDADE = V_EST_MES
			WHERE CDFILIAL = V_CDFILIAL AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND IDMODALIDADE = 'MES' AND NRORG = V_NRORG;
			
			/* Atualiza a estimativa da modalidade OUT */
			UPDATE METAMODALIDADEDIA SET VRPERCMODALIDADE = V_EST_OUT
			WHERE CDFILIAL = V_CDFILIAL AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND IDMODALIDADE = 'OUT' AND NRORG = V_NRORG;	
			
			FOR h IN 1..24 LOOP
				IF (V_VNDREALIZADAS > 0) THEN
					V_HORAS(h) := (V_HORAS(h)/ V_DIASDASEMANA)*100/V_VNDREALIZADAS;
				ELSE
					V_HORAS(h) := 0;
				END IF;				
				V_VND_PROV2 := V_VND_PROV2 + TRUNC(V_HORAS(h), 2);				
			END LOOP;	
			
			IF (V_VND_PROV2 < 100) THEN 
				V_HORAS(24) := V_HORAS(24) + (100-V_VND_PROV2);
			ELSIF (V_VND_PROV2 > 100) THEN 
				
				V_VND_PROV2 := V_VND_PROV2 - 100;
				
				FOR h IN 1..24 LOOP
					IF (V_HORAS(h) > V_VND_PROV2) THEN 
						V_HORAS(h) := V_HORAS(h) - V_VND_PROV2;	
						EXIT;
					END IF;
				END LOOP;
			END IF;
			
			V_CONTHORA := 0;
			
			WHILE V_CONTHORA <= 23 LOOP
				
				UPDATE METAHORA SET VRPERCHORA = TRUNC(V_HORAS(V_CONTHORA+1),2)
				WHERE CDFILIAL = V_CDFILIAL AND CDHORA = V_CONTHORA AND NRDIA = V_NRDIA AND CDMES = V_CDMES AND CDANO = V_CDANO AND NRORG = V_NRORG;	
				
				V_CONTHORA := V_CONTHORA + 1;
			END LOOP;
			
		/* END LOOP DE DIAS DA SEMANA */
		END LOOP; 
		
		IF(V_VENDASEMANA > 0) THEN
			FOR i IN 1..7 LOOP 
					V_VND_PROV := (V_VALORES(i)*100)/V_VENDASEMANA;
					UPDATE METADIA SET VRPERCDIA = TRUNC(V_VND_PROV,2), VRTICKETDIA = 100
					WHERE CDFILIAL = V_CDFILIAL AND NRDIA = i AND CDMES = V_CDMES AND CDANO = V_CDANO AND NRORG = V_NRORG;	
			END LOOP;
		END IF;
		
		UPDATE METAMES SET VRMES = TRUNC(V_TOTVENDAMES,2), VRSEM = TRUNC(V_VENDASEMANA,2)
		WHERE CDFILIAL = V_CDFILIAL AND CDMES = V_CDMES AND CDANO = V_CDANO AND NRORG = V_NRORG;

	/* END LOOP MES */
	END LOOP;
	COMMIT;
	CALCULA_METAS_RM(V_CDFILIAL, V_NRORG);
END;
/

/* PROCEDURE */
  create or replace PROCEDURE REPLICA_VENDAS_RM
AS
  V_VENDASPENDENTES NUMBER;
  V_REGISTROSVDM    NUMBER;
  V_REGISTROSVDT    NUMBER;
  V_NRPESSOASATEND  NUMBER;
  V_NRVENDACAN      NUMBER;
  V_NRVENDACON      NUMBER;
  V_CONTHORA        NUMBER;
  V_TOTALGERAL      FLOAT;
  V_TOTALVENDIDO    FLOAT;
  V_CDFILIAL        VARCHAR2(4);
  V_IDORIGVENDA     VARCHAR2(7);
  V_NRORG NUMBER;
BEGIN
  /* Verifica se existem vendas pendentes de verificao  */
  BEGIN
    SELECT COUNT(IDCONTVEND)
    INTO V_VENDASPENDENTES
    FROM VENDA
    WHERE IDCONTVEND = 'N'
    AND DTENTRVENDA  = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY');
  EXCEPTION
  WHEN NO_DATA_FOUND THEN
    V_VENDASPENDENTES := 0;
  END;
  /* Se encontrar alguma venda no passo anterior, realiza o passo de registr-las na tabela VENDADIAMODALIDADE */
  IF V_VENDASPENDENTES > 0 THEN
    FOR C_VENDAS IN
    (SELECT CDFILIAL, NMFILIAL, IDORIGEMVENDA, IDCONTVEND, DTENTRVENDA, NRORG,
       SUM(TOTALVEND) TOTALVEND,
       TRUNC(SUM(TICKETMEDIO) / SUM(DECODE(VRQTVENDACONC, 0, 1, VRQTVENDACONC)), 2) TICKETMEDIO,
       SUM(VRQTVENDACONC) VRQTVENDACONC, SUM(VRQTVENDACANC) VRQTVENDACANC
  FROM  (SELECT PARCIAL.CDFILIAL, PARCIAL.NMFILIAL, PARCIAL.IDORIGEMVENDA, PARCIAL.IDCONTVEND, PARCIAL.DTENTRVENDA, PARCIAL.NRORG,
                SUM(DECODE(PARCIAL.IDSITUVENDA, 'C', 0, PARCIAL.TOTALVEND)) TOTALVEND,
                TRUNC(SUM(PARCIAL.TICKETMEDIO)) AS TICKETMEDIO,
                SUM(COUNT(DISTINCT VRQTVENDACONC)) OVER (PARTITION BY CDFILIAL, IDORIGEMVENDA) VRQTVENDACONC,
                SUM(COUNT(DISTINCT VRQTVENDACANC)) OVER (PARTITION BY CDFILIAL, IDORIGEMVENDA) VRQTVENDACANC
           FROM (SELECT X.CDFILIAL, X.NMFILIAL, SUBSTR(X.IDORIGEMVENDA, 0, 3) AS  IDORIGEMVENDA, X.IDCONTVEND, X.DTENTRVENDA, X.NRORG,
                        DECODE(IDSITUVENDA, 'C', 0, TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+NVL(IT.VRUNITVENDCL,0)),2) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TOTALVEND,
                        DECODE(IDSITUVENDA, 'C', 0, TRUNC(IT.QTPRODVEND*(IT.VRUNITVEND+NVL(IT.VRUNITVENDCL,0)),2) - NVL(IT.VRDESITVEND,0) + NVL(IT.VRACRITVEND,0)) AS TICKETMEDIO,
                        X.IDSITUVENDA,
                        DECODE(IDSITUVENDA, 'O', X.CDFILIAL||X.CDCAIXA||X.NRSEQVENDA) VRQTVENDACONC,
                        DECODE(IDSITUVENDA, 'C', X.CDFILIAL||X.CDCAIXA||X.NRSEQVENDA) VRQTVENDACANC
                   FROM (SELECT DISTINCT V.CDFILIAL,
                                V.CDCAIXA,
                                V.NRSEQVENDA,
                                FLC.NMFILIAL,
                                V.IDORIGEMVENDA,
                                V.NRORG,
                                V.IDCONTVEND,
                                V.DTENTRVENDA,
                                V.IDSITUVENDA
                           FROM VENDA V, (SELECT DISTINCT CDFILIAL, NMFILIAL, CDLOJA, CDCAIXA FROM V_FILIALLOJACAIXA) FLC
                          WHERE V.CDCAIXA   = FLC.CDCAIXA
                            AND V.CDLOJA      = FLC.CDLOJA
                            AND V.CDFILIAL    = FLC.CDFILIAL
                            AND V.DTENTRVENDA = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY')
                            AND V.IDCONTVEND  = 'N'
                            AND V.IDSITUVENDA IN ('O', 'C')
                            ORDER BY V.NRSEQVENDA) X LEFT JOIN ITEMVENDA IT
                                                            ON X.CDFILIAL = IT.CDFILIAL
                                                           AND X.CDCAIXA    = IT.CDCAIXA
                                                           AND X.NRSEQVENDA = IT.NRSEQVENDA) PARCIAL
          GROUP BY PARCIAL.CDFILIAL, PARCIAL.NMFILIAL, PARCIAL.IDORIGEMVENDA, PARCIAL.IDCONTVEND, PARCIAL.DTENTRVENDA, PARCIAL.NRORG) FINAL
  GROUP BY CDFILIAL, NMFILIAL, IDORIGEMVENDA, IDCONTVEND, DTENTRVENDA, NRORG
  ORDER BY CDFILIAL
    )
    LOOP
      V_CDFILIAL     := C_VENDAS.CDFILIAL;
      V_IDORIGVENDA  := C_VENDAS.IDORIGEMVENDA;
      V_TOTALVENDIDO := C_VENDAS.TOTALVEND;
      V_NRORG        := C_VENDAS.NRORG;
	  V_NRVENDACAN   := C_VENDAS.VRQTVENDACANC;
	  V_NRVENDACON   := C_VENDAS.VRQTVENDACONC;
	  
      /* Verifica se o registro j possui campo na tabela VENDADIAMODALIDADE */
      BEGIN
        SELECT COUNT(CDFILIAL)
        INTO V_REGISTROSVDM
        FROM VENDADIAMODALIDADE VDM
        WHERE VDM.CDFILIAL   = V_CDFILIAL
        AND VDM.DTVENDA      = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY')
        AND VDM.IDMODALIDADE = V_IDORIGVENDA;
      EXCEPTION
      WHEN NO_DATA_FOUND THEN
        V_VENDASPENDENTES := 0;
      END;
      /* Caso no exista, cria. Se existe, atualiza */
      IF V_REGISTROSVDM = 0 THEN
        INSERT
        INTO VENDADIAMODALIDADE
          ( CDFILIAL, DTVENDA, IDMODALIDADE, VRTOTAL, NRORG )
          VALUES
          ( V_CDFILIAL, TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY'),
            V_IDORIGVENDA, V_TOTALVENDIDO, V_NRORG );
      ELSE
        UPDATE VENDADIAMODALIDADE
        SET VRTOTAL      = VRTOTAL + V_TOTALVENDIDO
        WHERE DTVENDA    = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
        AND CDFILIAL     = V_CDFILIAL
        AND IDMODALIDADE = V_IDORIGVENDA;
      END IF;
	  
	  /* Verifica se o registro j possui campo na tabela VENDADIATOTAL */
      BEGIN
        SELECT COUNT(CDFILIAL)
        INTO V_REGISTROSVDT
        FROM VENDADIATOTAL VDT
        WHERE VDT.CDFILIAL   = V_CDFILIAL
        AND VDT.DTVENDA      = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY');
      EXCEPTION
      WHEN NO_DATA_FOUND THEN
        V_VENDASPENDENTES := 0;
      END;
	  
	  /* Caso no exista, cria. Se existe, atualiza */
      IF V_REGISTROSVDT = 0 THEN
        INSERT
        INTO VENDADIATOTAL
          ( CDFILIAL, DTVENDA, NRVENDACON, NRVENDACAN, VRTICKETMED, VRTOTAL, NRORG )
          VALUES
          ( V_CDFILIAL, TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY'),
			V_NRVENDACON, V_NRVENDACAN, TRUNC(V_TOTALVENDIDO/DECODE(V_NRVENDACON, 0, 1, V_NRVENDACON),2),
            V_TOTALVENDIDO, V_NRORG );
      ELSE
        UPDATE VENDADIATOTAL
        SET NRVENDACON = NRVENDACON+V_NRVENDACON,
			 NRVENDACAN = NRVENDACAN+V_NRVENDACAN,
			 VRTICKETMED = TRUNC((VRTOTAL+V_TOTALVENDIDO)/DECODE(NRVENDACON+V_NRVENDACON, 0, 1, NRVENDACON+V_NRVENDACON), 2),
             VRTOTAL = VRTOTAL+V_TOTALVENDIDO
        WHERE DTVENDA    = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'))
        AND CDFILIAL     = V_CDFILIAL;
      END IF;	
	  
	  /* Calcula venda por hora */
	  VENDA_HORA(V_CDFILIAL, V_NRORG);
	  
    END LOOP;    
	  /* Marca a venda como registrada */
      UPDATE VENDA
      SET IDCONTVEND    = 'S'
      WHERE IDSITUVENDA IN ('O', 'C')
      AND DTENTRVENDA   = TO_DATE(TO_CHAR(sysdate, 'DD/MM/YYYY'), 'DD/MM/YYYY');	
	  
	  COMMIT;
  END IF;
END;
/

-------------------------------------------------- JOB -------------------------------------
/* JOB */
  BEGIN
   DECLARE
      V_COUNT NUMBER;
   BEGIN
      V_COUNT := 0;
      BEGIN
         SELECT COUNT(*)
           INTO V_COUNT
           FROM USER_SCHEDULER_JOBS
          WHERE JOB_NAME = 'JOB_REPLICA_VENDAS_RM';
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_COUNT := 0;
      END;
      IF V_COUNT = 0 THEN
         DBMS_SCHEDULER.CREATE_JOB (
            job_name        => 'JOB_REPLICA_VENDAS_RM',
            job_type        => 'PLSQL_BLOCK',
            job_action      => 'BEGIN REPLICA_VENDAS_RM(); END;',
            start_date      => SYSDATE,
            repeat_interval => 'FREQ=MINUTELY;INTERVAL=10',
            end_date        => NULL,
            enabled         => TRUE,
            comments        => 'Job criado para atualizar vendas das tabelas VENDADIAMODALIDADE e VENDADIATOTAL usando no Retail Manage.'
         );
      ELSE
         DBMS_SCHEDULER.DROP_JOB(JOB_NAME => 'JOB_REPLICA_VENDAS_RM');
         DBMS_SCHEDULER.CREATE_JOB (
            job_name        => 'JOB_REPLICA_VENDAS_RM',
            job_type        => 'PLSQL_BLOCK',
            job_action      => 'BEGIN REPLICA_VENDAS_RM(); END;',
            start_date      => SYSDATE,
            repeat_interval => 'FREQ=MINUTELY;INTERVAL=10',
            end_date        => NULL,
            enabled         => TRUE,
            comments        => 'Job criado para atualizar vendas das tabelas VENDADIAMODALIDADE e VENDADIATOTAL usando no Retail Manage.'
         );
      END IF;
   END;
END;
/






